/*
 * MemoryStatusPanel.java
 * =================================================================
 * Copyright (C) 2009 Shawn E. Gano
 *
 * This file is part of JSatTrak.
 *
 * JSatTrak is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * JSatTrak is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with JSatTrak.  If not, see <http://www.gnu.org/licenses/>.
 * ====================================================================
 *
 * Created on August 2, 2008, 10:18 PM
 */

package name.gano.swingx.memstatus;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.text.DecimalFormat;
import javax.swing.Timer;

/**
 *
 * @author  Shawn
 */
public class MemoryStatusPanel extends javax.swing.JPanel implements MouseListener
{
    private Timer playTimer;
    private boolean stopTimer = false;
    private int updateRateMs = 2500;
    
    DecimalFormat df = new DecimalFormat("#,##0.00");

    /** Creates new form MemoryStatusPanel */
    public MemoryStatusPanel() 
    {
        initComponents();
        
        this.addMouseListener(this);
        
        memProgressBar.setMaximum(100);
        memProgressBar.addMouseListener(this);
        
        playTimer = new Timer(updateRateMs, new ActionListener()
        {
            public void actionPerformed(ActionEvent evt)
            {
                updateStatus();
                //System.out.println("Mem update");    
                
                if (stopTimer)
                {
                    playTimer.stop();
                    //System.out.println("Mem stopped!");                                     
                }
            }
        });
        playTimer.setRepeats(true);
        playTimer.start();
        
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        memProgressBar = new javax.swing.JProgressBar();

        memProgressBar.setString("0 MB / 0 MB");
        memProgressBar.setStringPainted(true);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(memProgressBar, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(memProgressBar, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );
    }// </editor-fold>//GEN-END:initComponents

    public void updateStatus()
    {
        double totMem1 = java.lang.Runtime.getRuntime().totalMemory() / 1048576.0; // Megabytes
        double freeMem1 = java.lang.Runtime.getRuntime().freeMemory() / 1048576.0;
        double usedMem1 = totMem1 - freeMem1;

        // take one time step in the aimation
        int usedMemPercent = (int) Math.round(100.0 * (usedMem1) / totMem1);
        memProgressBar.setValue(usedMemPercent);

        memProgressBar.setString(df.format(usedMem1) + " MB / " + df.format(totMem1) + " MB");
        
        memProgressBar.setToolTipText("Free Memory: " + df.format(freeMem1) + " MB");
        
        if(!memProgressBar.isShowing())
        {
            stopTimer = true; // stop updates
            memProgressBar.setString("STOPPED - click to restart");
        }
    } // updateStatus
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JProgressBar memProgressBar;
    // End of variables declaration//GEN-END:variables


    public void mousePressed(MouseEvent e)
    {
    }

    public void mouseReleased(MouseEvent e)
    {
    }

    public void mouseEntered(MouseEvent e)
    {
    }

    public void mouseExited(MouseEvent e)
    {
    }

    public void mouseClicked(MouseEvent e)
    {
        System.gc();
        updateStatus();
        //System.out.println("GC");
        // if stopped - restart
        if(stopTimer == true)
        {
            stopTimer = false; 
            playTimer.start();
        }
    }
    

}
